import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:artemis_cupps/artemis_cupps.dart';

class LogScreen extends StatefulWidget {
  const LogScreen({Key? key}) : super(key: key);

  @override
  _LogScreenState createState() => _LogScreenState();
}

class _LogScreenState extends State<LogScreen> {
  String logType = "platform";
  String logDate = DateTime.now().toString().substring(0, 10);

  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;

    return Scaffold(
      appBar: AppBar(
        title: const Text("LOG"),
        actions: [
          IconButton(
              onPressed: () {
                setState(() {});
              },
              icon: const Icon(Icons.refresh))
        ],
      ),
      body: Container(
        padding: const EdgeInsets.symmetric(horizontal: 0, vertical: 12),
        child: Column(
          children: [
            Row(
              children: [
                Expanded(
                    child: IconButton(
                  icon: const Icon(Icons.chevron_left),
                  onPressed: () {
                    DateTime prev = DateTime.parse(logDate).add(const Duration(days: -1));
                    setState(() {
                      logDate = prev.toString().substring(0, 10);
                    });
                  },
                )),
                Expanded(
                    child: TextButton(
                  child: Text(logDate),
                  onPressed: () {},
                )),
                Expanded(
                    child: IconButton(
                  icon: const Icon(Icons.chevron_right),
                  onPressed: () {
                    DateTime prev = DateTime.parse(logDate).add(const Duration(days: 1));
                    setState(() {
                      logDate = prev.toString().substring(0, 10);
                    });
                  },
                )),
              ],
            ),
            const Divider(),
            SizedBox(
              width: width,
              child: CupertinoSegmentedControl(
                  groupValue: logType,
                  children: const {
                    "platform": Text("Plat"),
                    "bp": Text("Bp"),
                    "bt": Text("Bt"),
                    "pr": Text("Pr"),
                    "bg": Text("Bg"),
                    "bc": Text("Bc"),
                    "oc": Text("Oc"),
                    "ms": Text("Ms"),
                  },
                  onValueChanged: (String v) {
                    setState(() {
                      logType = v;
                    });
                  }),
            ),
            const Divider(),
            Expanded(
              child: FutureBuilder(
                  future: Logger.getLogList(),
                  builder: (c, snapshot) {
                    if (snapshot.connectionState == ConnectionState.done || snapshot.hasData) {
                      List<LogEntity> data = (snapshot.data as List<LogEntity>)
                          .where((d) => d.type.toLowerCase() == logType.toLowerCase()&& d.dateStr == logDate )
                          .toList();
                      return ListView(
                        children: data
                            .map((e) => ListTile(
                                  title: Text(e.name),
                                  subtitle: Row(
                                    mainAxisAlignment: MainAxisAlignment.end,
                                    children: [
                                      SizedBox(
                                        width: 50,
                                        child: IconButton(
                                            onPressed: () {
                                              e.download();
                                            },
                                            icon: const Icon(
                                              Icons.download,
                                              color: Colors.blueAccent,
                                            )),
                                      ),
                                      SizedBox(width: 24),
                                      SizedBox(
                                        width: 50,
                                        child: IconButton(
                                            onPressed: () {
                                              e.clear();
                                              setState(() {});
                                            },
                                            icon: const Icon(
                                              Icons.delete,
                                              color: Colors.red,
                                            )),
                                      ),
                                    ],
                                  ),
                                ))
                            .toList(),
                      );
                    }
                    return const Text("Loading ip");
                  }),
            ),
          ],
        ),
      ),
    );
  }
}
