import 'package:flutter/material.dart';
import 'package:artemis_cupps/artemis_cupps.dart';

class OcDeviceScreen extends StatefulWidget {
  final OcDevice device;

  const OcDeviceScreen({required this.device, Key? key}) : super(key: key);

  @override
  _OcDeviceScreenState createState() => _OcDeviceScreenState();
}

class _OcDeviceScreenState extends State<OcDeviceScreen> {
  List<OcData> readData = [];

  @override
  void initState() {
    // TODO: implement initState
    CUPPS.setOcListener((result) {
      setState(() {
        readData += result;
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;
    OcDevice device = widget.device;
    return Scaffold(
      appBar: AppBar(
        title: Text("${device.deviceIndex} ${device.deviceName}"),
        actions: [
          IconButton(
              onPressed: () {
                device.refreshStatus();
              },
              icon: const Icon(Icons.refresh))
        ],
      ),
      body: Container(
        padding: const EdgeInsets.symmetric(horizontal: 12, vertical: 12),
        child: Column(
          children: [
            Row(
              children: [
                Expanded(flex: 1, child: Image.asset(device.deviceParameter.status.image)),
                const SizedBox(width: 12),
                Expanded(
                    flex: 2,
                    child: Column(
                      children: [
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            const Text("Status"),
                            Text(
                              "${device.deviceParameter.status}",
                              style: const TextStyle(fontWeight: FontWeight.bold),
                            ),
                          ],
                        ),
                        const Divider(),
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            const Text("Address"),
                            Text(
                              "${device.deviceParameter.ipAndPort}",
                              style: const TextStyle(fontWeight: FontWeight.bold),
                            ),
                          ],
                        ),
                        const Divider(),
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [const Text("TCP Connection"), Icon(Icons.circle, color: device.isConnected ? Colors.green : Colors.red, size: 14)],
                        ),
                      ],
                    ))
              ],
            ),
            Row(
              children: [
                const Text("Available Modes"),
                const SizedBox(width: 12),
                Expanded(
                  child: Wrap(
                    direction: Axis.horizontal,
                    children: device.supportedInterfaceModes.map((e) => OutlinedButton(onPressed: () {}, child: Text(e.mode))).toList(),
                  ),
                ),
              ],
            ),
            const Divider(),
            ElevatedButton(
              onPressed: () {
                device.lockDevice();
              },
              child: const Text("Lock Device"),
            ),
            ElevatedButton(
              onPressed: () {
                device.unlockDevice();
              },
              child: const Text("Unlock Device"),
            ),
            Expanded(
                child: ListView(
              children: readData.map((e) => Text("$e")).toList(),
            )),
          ],
        ),
      ),
    );
  }
}
