import '../../classes/device.dart';
import 'package:xml/xml.dart';
import '../classes.dart';

class DdDevice extends Device {
  DdDevice(
      {required DdDeviceParameter deviceParameter,
      required String deviceIndex,
      required String deviceName,
      required String deviceParameterType,
      required List<InterfaceMode> supportedInterfaceModes,
      required List<Device> devices})
      : super(
            deviceParameter: deviceParameter,
            devices: devices,
            deviceParameterType: deviceParameterType,
            deviceName: deviceName,
            deviceIndex: deviceIndex,
            supportedInterfaceModes: supportedInterfaceModes);

  factory DdDevice.fromDevice(Device d) {
    return DdDevice(
      deviceParameter: DdDeviceParameter.fromXML(d.deviceParameter.toXML()),
      deviceIndex: d.deviceIndex,
      deviceName: d.deviceName,
      deviceParameterType: d.deviceParameterType,
      supportedInterfaceModes: d.supportedInterfaceModes,
      devices: d.devices,
    );
  }
}

class DdDeviceParameter extends DeviceParameter {
  final String numCharsPerLine;
  final String numLines;
  final String supportsAnimation;
  final String pixelsTall;
  final String pixelsWide;

  DdDeviceParameter({
    required VendorModelInfo vendorModelInfo,
    required IpAndPort ipAndPort,
    required Status status,
    required this.numCharsPerLine,
    required this.numLines,
    required this.supportsAnimation,
    required this.pixelsTall,
    required this.pixelsWide,
  }) : super(
            vendorModelInfo: vendorModelInfo,
            ipAndPort: ipAndPort,
            status: status);

  factory DdDeviceParameter.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    XmlElement? vendorModelInfoElement =
        document.firstChild!.getElement("vendorModelInfo");
    XmlElement? ipAndPortElement = document.firstChild!.getElement("ipAndPort");
    XmlElement? statusElement = document.firstChild!.getElement("ddStatus");

    VendorModelInfo vendorModelInfo =
        VendorModelInfo.fromXML(vendorModelInfoElement!.toXmlString());
    IpAndPort ipAndPort = IpAndPort.fromXML(ipAndPortElement!.toXmlString());
    DdStatus status = DdStatus.fromXML(statusElement!.toXmlString());

    String numCharsPerLine = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "numCharsPerLine")
        .value;
    String numLines = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "numLines")
        .value;
    String supportsAnimation = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "supportsAnimation")
        .value;
    String pixelsTall = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "pixelsTall")
        .value;
    String pixelsWide = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "pixelsWide")
        .value;
    return DdDeviceParameter(
      vendorModelInfo: vendorModelInfo,
      ipAndPort: ipAndPort,
      status: status,
      numCharsPerLine: numCharsPerLine,
      numLines: numLines,
      supportsAnimation: supportsAnimation,
      pixelsTall: pixelsTall,
      pixelsWide: pixelsWide,
    );
  }

  @override
  String toXML() {
    return '''<ddDeviceParameter numCharsPerLine="$numCharsPerLine" numLines="$numLines" supportsAnimation="$supportsAnimation" pixelsTall="$pixelsTall" pixelsWide="$pixelsWide" >
    ${vendorModelInfo.toXML()}
    ${ipAndPort.toXML()}
    ${status.toXML()}
    </ddDeviceParameter>
    ''';
  }
}

class DdStatus extends Status {
  DdStatus({
    required String desc,
    required String init,
    required String powerOff,
    required String ready,
    required String unknown,
  }) : super(
            desc: desc,
            init: init,
            powerOff: powerOff,
            ready: ready,
            unknown: unknown);

  factory DdStatus.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String desc = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "desc",
            orElse: orElseEmpty)
        .value;
    String init = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "init")
        .value;
    String powerOff = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "powerOff")
        .value;
    String ready = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "ready")
        .value;
    String unknown = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "unknown")
        .value;
    return DdStatus(
        desc: desc,
        init: init,
        powerOff: powerOff,
        ready: ready,
        unknown: unknown);
  }

  @override
  String toXML() {
    return '''<ddStatus desc="$desc" init="$init" powerOff="$powerOff" ready="$ready" unknown="$unknown"/>''';
  }
}
