import '../../classes/device.dart';
import 'package:xml/xml.dart';
import '../classes.dart';

class ZlDevice extends Device {
  ZlDevice(
      {required ZlDeviceParameter deviceParameter,
      required String deviceIndex,
      required String deviceName,
      required String deviceParameterType,
      required List<InterfaceMode> supportedInterfaceModes,
      required List<Device> devices})
      : super(
            deviceParameter: deviceParameter,
            devices: devices,
            deviceParameterType: deviceParameterType,
            deviceName: deviceName,
            deviceIndex: deviceIndex,
            supportedInterfaceModes: supportedInterfaceModes);

  factory ZlDevice.fromDevice(Device d) {
    return ZlDevice(
      deviceParameter: ZlDeviceParameter.fromXML(d.deviceParameter.toXML()),
      deviceIndex: d.deviceIndex,
      deviceName: d.deviceName,
      deviceParameterType: d.deviceParameterType,
      supportedInterfaceModes: d.supportedInterfaceModes,
      devices: d.devices,
    );
  }
}

class ZlDeviceParameter extends DeviceParameter {
  ZlDeviceParameter({
    required VendorModelInfo vendorModelInfo,
    required IpAndPort ipAndPort,
    required Status status,
  }) : super(
            vendorModelInfo: vendorModelInfo,
            ipAndPort: ipAndPort,
            status: status);

  factory ZlDeviceParameter.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    XmlElement? vendorModelInfoElement =
        document.firstChild!.getElement("vendorModelInfo");
    XmlElement? ipAndPortElement = document.firstChild!.getElement("ipAndPort");
    XmlElement? statusElement = document.firstChild!.getElement("zlStatus");

    VendorModelInfo vendorModelInfo = vendorModelInfoElement == null
        ? VendorModelInfo(vendor: '', model: '', miscInfo: '')
        : VendorModelInfo.fromXML(vendorModelInfoElement.toXmlString());
    IpAndPort ipAndPort = IpAndPort.fromXML(ipAndPortElement!.toXmlString());
    ZlStatus status = ZlStatus.fromXML(statusElement!.toXmlString());

    return ZlDeviceParameter(
      vendorModelInfo: vendorModelInfo,
      ipAndPort: ipAndPort,
      status: status,
    );
  }

  @override
  String toXML() {
    return '''<zlDeviceParameter>
    ${ipAndPort.toXML()}
    ${status.toXML()}
    </zlDeviceParameter>
    ''';
  }
}

class ZlStatus extends Status {
  ///instead of powerOff it has diskError///
  ZlStatus({
    required String desc,
    required String init,
    required String diskError,
    required String ready,
    required String unknown,
  }) : super(
            desc: desc,
            init: init,
            powerOff: diskError,
            ready: ready,
            unknown: unknown);

  factory ZlStatus.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String desc = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "desc",
            orElse: orElseEmpty)
        .value;
    String init = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "init")
        .value;
    String diskError = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "diskError")
        .value;
    String ready = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "ready")
        .value;
    String unknown = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "unknown")
        .value;
    return ZlStatus(
        desc: desc,
        init: init,
        diskError: diskError,
        ready: ready,
        unknown: unknown);
  }

  @override
  String toXML() {
    return '''<zlStatus desc="$desc" init="$init" diskError="$powerOff" ready="$ready" unknown="$unknown"/>''';
  }
}
