import '../../classes/notify.dart';
import 'package:xml/xml.dart';
import '../classes.dart';
import '../device.dart';

class DErrEnteredEvent extends Notify {
  final Status status;
  final String device;
  final String deviceType;

  DErrEnteredEvent({
    required int messageID,
    required String messageName,
    required this.status,
    required this.device,
    required this.deviceType,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "dErrEnteredEvent");

  factory DErrEnteredEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("dErrEnteredEvent").first;
    String device = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "device", orElse: orElseEmpty).value;
    String deviceType = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "deviceType", orElse: orElseEmpty).value;
    XmlElement statusElement = eventElement.childElements.where((element) => element.name.toString().endsWith("Status")).first;

    Status status = Status.fromXML(statusElement.toXmlString());
    return DErrEnteredEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      status: status,
      device: device,
      deviceType: deviceType
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <dErrEnteredEvent device="$device" deviceType="$deviceType">
                ${status.toXML()}
            </dErrEnteredEvent>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
