import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class DevicePollingTooFastEvent extends Notify {
  final String applicationName;
  final String applicationVersion;

  DevicePollingTooFastEvent({
    required int messageID,
    required String messageName,
    required this.applicationVersion,
    required this.applicationName,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "devicePollingTooFastEvent");

  factory DevicePollingTooFastEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("devicePollingTooFastEvent").first;
    String applicationVersion = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationVersion", orElse: orElseEmpty).value;
    String applicationName = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationName", orElse: orElseEmpty).value;

    return DevicePollingTooFastEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      applicationName: applicationName,
      applicationVersion: applicationVersion,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <devicePollingTooFastEvent applicationName="$applicationName" applicationVersion="$applicationVersion"/>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
