import '../../classes/notify.dart';
import 'package:xml/xml.dart';
import '../classes.dart';
import '../device.dart';

class DStpExitedEvent extends Notify {
  final String device;

  DStpExitedEvent({
    required int messageID,
    required String messageName,
    required this.device,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "dStpExitedEvent");

  factory DStpExitedEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("dStpExitedEvent").first;
    String device = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "device", orElse: orElseEmpty).value;

    return DStpExitedEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      device: device,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <dStpExitedEvent device="$device"/>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
