import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class ExceptionEvent extends Notify {
  final String exceptionSource;
  final String user;
  final String device;
  final String workstation;
  final String applicationName;
  final String applicationVersion;
  final String arg;
  final String body;

  ExceptionEvent({
    required int messageID,
    required String messageName,
    required this.exceptionSource,
    required this.user,
    required this.device,
    required this.arg,
    required this.workstation,
    required this.applicationVersion,
    required this.applicationName,
    required this.body,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "exceptionEvent");

  factory ExceptionEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("exceptionEvent").first;
    String user = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "user", orElse: orElseEmpty).value;
    String device = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "device", orElse: orElseEmpty).value;
    String exceptionSource = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "exceptionSource", orElse: orElseEmpty).value;
    String arg = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "arg", orElse: orElseEmpty).value;
    String applicationVersion = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationVersion", orElse: orElseEmpty).value;
    String applicationName = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationName", orElse: orElseEmpty).value;
    String workstation =
        eventElement.attributes.firstWhere((p0) => p0.name.toString() == "workstation", orElse: orElseEmpty).value;
    String body = eventElement.text;
    return ExceptionEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      user: user,
      device: device,
      arg: arg,
      exceptionSource: exceptionSource,
      workstation: workstation,
      applicationName: applicationName,
      applicationVersion: applicationVersion,
      body: body
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <exceptionEvent exceptionSource="$exceptionSource" applicationName="$applicationName" 
            applicationVersion="$applicationVersion" device="$device" user="$user" workstation="$workstation" arg="$arg">
            $body
            </exceptionEvent>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
