import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class IllogicalMessageErrorEvent extends Notify {
  final String user;
  final String workstation;
  final String applicationName;
  final String applicationVersion;
  final String illogicalMessage;
  final List<String> expectedMessageNameList;

  IllogicalMessageErrorEvent({
    required int messageID,
    required String messageName,
    required this.user,
    required this.workstation,
    required this.applicationVersion,
    required this.applicationName,
    required this.illogicalMessage,
    required this.expectedMessageNameList,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "illogicalMessageErrorEvent");

  factory IllogicalMessageErrorEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("illogicalMessageErrorEvent").first;
    String user = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "user", orElse: orElseEmpty).value;
    String applicationVersion = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationVersion", orElse: orElseEmpty).value;
    String applicationName = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationName", orElse: orElseEmpty).value;
    String workstation =
        eventElement.attributes.firstWhere((p0) => p0.name.toString() == "workstation", orElse: orElseEmpty).value;
    XmlElement illogicalMessageElement = eventElement.findAllElements("illogicalMessage").first;
    String illogicalMessage = illogicalMessageElement.text;
    XmlElement expectedMessageNameListElement = eventElement.findAllElements("expectedMessageNameList").first;
    List<String> expectedMessageNameList = expectedMessageNameListElement.childElements.map((e) => e.attributes.firstWhere((p0)
    => p0.name.toString()=="messageName",orElse: orElseEmpty).value).toList();


    return IllogicalMessageErrorEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      user: user,
      workstation: workstation,
      applicationName: applicationName,
      applicationVersion: applicationVersion,
      illogicalMessage:illogicalMessage,
      expectedMessageNameList: expectedMessageNameList
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <illogicalMessageErrorEvent applicationName="$applicationName" 
            applicationVersion="$applicationVersion" user="$user" workstation="$workstation">
                <illogicalMessage>
                    $illogicalMessage
                </illogicalMessage>
                <expectedMessageNameList>
                    ${expectedMessageNameList.map((e) => '<messageName messageName="$e"/>').join()}
                </expectedMessageNameList>
            </illogicalMessageErrorEvent>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
