import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class InterfaceLevelErrorEvent extends Notify {
  final String user;
  final String device;
  final String workstation;
  final String applicationName;
  final String applicationVersion;
  final String errorType;

  InterfaceLevelErrorEvent({
    required int messageID,
    required String messageName,
    required this.errorType,
    required this.user,
    required this.device,
    required this.workstation,
    required this.applicationVersion,
    required this.applicationName,
  }) : super(
            messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "interfaceLevelErrorEvent");

  factory InterfaceLevelErrorEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    XmlElement eventElement = document.rootElement.findAllElements("interfaceLevelErrorEvent").first;
    String user = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "user", orElse: orElseEmpty).value;
    String device = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "device", orElse: orElseEmpty).value;
    String errorType = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "errorType", orElse: orElseEmpty).value;
    String applicationVersion = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationVersion", orElse: orElseEmpty).value;
    String applicationName = eventElement.attributes.firstWhere((p0) => p0.name.toString() == "applicationName", orElse: orElseEmpty).value;
    String workstation =
        eventElement.attributes.firstWhere((p0) => p0.name.toString() == "workstation", orElse: orElseEmpty).value;
    return InterfaceLevelErrorEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
      user: user,
      device: device,
      workstation: workstation,
      applicationName: applicationName,
      applicationVersion: applicationVersion,
      errorType: errorType
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <notify>
            <interfaceLevelErrorEvent applicationName="$applicationName" 
            applicationVersion="$applicationVersion" device="$device" user="$user" workstation="$workstation" errorType="$errorType"/>
        </notify>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
