import 'package:artemis_cupps/artemis_cupps.dart';
import '../../classes/notify.dart';
import 'package:xml/xml.dart';

class MessageIDErrorEvent extends Notify {
  MessageIDErrorEvent({
    required int messageID,
    required String messageName,
  }) : super(messageID: messageID, messageName: messageName, notificationType: NotificationType.unknown, name: "messageIDErrorEvent");

  factory MessageIDErrorEvent.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageName").value;
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;

    return MessageIDErrorEvent(
      messageName: messageName,
      messageID: int.parse(messageID),
    );
  }

  @override
  String toXML() {
    return '''<?xml version="1.0"?>
<cupps xmlns="http://www.cupps.aero/cupps/01.03" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" messageID="$messageID" 
messageName="notify">
  <notify>
    <messageIDErrorEvent errorType="messageIDSequenceError"></messageIDErrorEvent>
  </notify>
</cupps>'''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
