import 'package:xml/xml.dart';

import '../classes.dart';

class ApplicationStopCommandRequest extends CuppsBase {
  final String stopMessage;
  final String canDefer;

  ApplicationStopCommandRequest({
    String? xmlns,
    String? xsi,
    required int messageID,
    required this.stopMessage,
    required this.canDefer,
  }) : super(
            messageID: messageID, messageName: "applicationStopCommandRequest");

  factory ApplicationStopCommandRequest.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String stopMessage = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "stopMessage",
            orElse: orElseEmpty)
        .value;
    String canDefer = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "canDefer",
            orElse: orElseEmpty)
        .value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;

    return ApplicationStopCommandRequest(
        canDefer: canDefer,
        stopMessage: stopMessage,
        messageID: int.parse(messageID));
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
    <$messageName stopMessage="$stopMessage" canDefer="$canDefer"/>
    </cupps>
    ''';
  }

  @override
  String toString() {
    return "$messageID : $messageName : $xsi : $stopMessage : $xmlns : $canDefer";
  }
}
