import 'package:xml/xml.dart';

import '../classes.dart';

class DeviceAcquireRequest extends CuppsBase {
  final String deviceName;
  final String deviceToken;
  final String airlineID;
  final String msFoidMasking;

  DeviceAcquireRequest({
    String? xmlns,
    String? xsi,
    required int messageID,
    required this.deviceName,
    required this.deviceToken,
    required this.airlineID,
    required this.msFoidMasking,
  }) : super(messageID: messageID, messageName: "deviceAcquireRequest");

  factory DeviceAcquireRequest.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String deviceName = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "deviceName",
            orElse: orElseEmpty)
        .value;
    String deviceToken = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "deviceToken",
            orElse: orElseEmpty)
        .value;
    String airlineID = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "airlineID",
            orElse: orElseEmpty)
        .value;
    String msFoidMasking = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "msFoidMasking",
            orElse: orElseEmpty)
        .value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;

    return DeviceAcquireRequest(
        deviceName: deviceName,
        deviceToken: deviceToken,
        airlineID: airlineID,
        msFoidMasking: msFoidMasking,
        messageID: int.parse(messageID));
  }

  @override
  String toXML() {
    return '''<?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
    <$messageName deviceName="$deviceName" deviceToken="$deviceToken" airlineID="$airlineID" />
    </cupps>''';
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
