import 'package:xml/xml.dart';

import '../classes.dart';

class DeviceLockRequest extends CuppsBase {
  final String lockMethod;
  final String renew;

  DeviceLockRequest({
    required int messageID,
    String? xmlns,
    String? xsi,
    required this.lockMethod,
    required this.renew,
  }) : super(messageID: messageID, messageName: "deviceLockRequest");

  factory DeviceLockRequest.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String lockMethod = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "lockMethod",
            orElse: orElseEmpty)
        .value;
    String renew = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "renew", orElse: orElseEmpty)
        .value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;

    return DeviceLockRequest(
        lockMethod: lockMethod, messageID: int.parse(messageID), renew: renew);
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <$messageName lockMethod="$lockMethod" renew="$renew"/>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
