import 'package:xml/xml.dart';

import '../classes.dart';

class InterfaceLevelRequest extends CuppsBase {
  final String level;

  InterfaceLevelRequest({
    required int messageID,
    String? xmlns,
    String? xsi,
    required this.level,
  }) : super(messageID: messageID, messageName: "interfaceLevelRequest");

  factory InterfaceLevelRequest.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;

    String xsi = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "xmlns:xsi")
        .value;
    String level = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "level")
        .value;

    return InterfaceLevelRequest(
        level: level, xsi: xsi, messageID: int.parse(messageID));
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <$messageName level="$level"/>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName : $xsi : $level";
  }
}
