import 'package:xml/xml.dart';

import '../classes.dart';

class InterfaceModeRequest extends CuppsBase {
  final String mode;

  InterfaceModeRequest({
    required int messageID,
    String? xmlns,
    String? xsi,
    required this.mode,
  }) : super(messageID: messageID, messageName: "interfaceModeRequest");

  factory InterfaceModeRequest.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String mode = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "mode", orElse: orElseEmpty)
        .value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;

    return InterfaceModeRequest(mode: mode, messageID: int.parse(messageID));
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <$messageName mode="$mode" />
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName : $mode";
  }
}
