import 'package:xml/xml.dart';
import '../classes.dart';
import '../device.dart';

class DeviceAcquireResponse extends CuppsBase {
  final String result;
  final Device? device;

  DeviceAcquireResponse({
    required int messageID,
    required String messageName,
    required this.result,
    required this.device,
  }) : super(messageID: messageID, messageName: messageName);

  factory DeviceAcquireResponse.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageName")
        .value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;
    String result = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "result")
        .value;
    XmlElement? deviceElement =
        document.rootElement.findAllElements("device").isEmpty
            ? null
            : document.rootElement.findAllElements("device").first;
    Device? device = deviceElement == null
        ? null
        : Device.fromXML(deviceElement.toXmlString());

    return DeviceAcquireResponse(
        messageName: messageName,
        messageID: int.parse(messageID),
        result: result,
        device: device);
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <$messageName result="$result">
        ${device == null ? "" : device!.toXML()}
        </$messageName>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName : $xsi : $result : $xmlns ";
  }
}
