import '../../classes/Requests/authenticate_req.dart';
import 'package:xml/xml.dart';
import '../classes.dart';
import '../device.dart';

class DeviceLockResponse extends CuppsBase {
  final String result;
  final DeviceLockerInfo? deviceLockerInfo;

  DeviceLockResponse(
      {required int messageID,
      required String messageName,
      required this.result,
      required this.deviceLockerInfo})
      : super(messageID: messageID, messageName: messageName);

  factory DeviceLockResponse.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageName")
        .value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;
    String result = document.rootElement.childElements.first.attributes
        .firstWhere((p0) => p0.name.toString() == "result")
        .value;
    XmlElement? deviceLockerInfoElement =
        document.rootElement.firstChild!.getElement("deviceLockerInfo");
    DeviceLockerInfo? deviceLockerInfo = deviceLockerInfoElement == null
        ? null
        : DeviceLockerInfo.fromXML(deviceLockerInfoElement.toXmlString());

    return DeviceLockResponse(
        messageName: messageName,
        messageID: int.parse(messageID),
        result: result,
        deviceLockerInfo: deviceLockerInfo);
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        ${deviceLockerInfo == null ? '<$messageName result="$result"/>' : '<$messageName result="$result">${deviceLockerInfo!.toXML()}</$messageName>'}
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName: $result";
  }
}

class DeviceLockerInfo {
  final String lockMethod;
  final IpAndPort ipAndPort;
  final List<Application> applicationList;

  DeviceLockerInfo(
      {required this.lockMethod,
      required this.ipAndPort,
      required this.applicationList});

  factory DeviceLockerInfo.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    XmlElement? ipAndPortElement = document.firstChild!.getElement("ipAndPort");
    XmlElement? applicationListElement =
        document.firstChild!.getElement("applicationList");
    List<XmlElement> applications =
        applicationListElement!.findElements("application").toList();
    IpAndPort ipAndPort = IpAndPort.fromXML(ipAndPortElement!.toXmlString());
    String lockMethod = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "lockMethod")
        .value;
    List<Application> applicationList =
        applications.map((e) => Application.fromXML(e.toXmlString())).toList();
    return DeviceLockerInfo(
        ipAndPort: ipAndPort,
        lockMethod: lockMethod,
        applicationList: applicationList);
  }

  String toXML() {
    return '''
    <deviceLockerInfo lockMethod="$lockMethod" >
    ${ipAndPort.toXML()}
    <applicationList>
    ${applicationList.map((e) => e.toXML()).join()}
    </applicationList>
    </deviceLockerInfo>
    ''';
  }
}
