import 'dart:async';
import 'dart:developer';
import 'package:artemis_cupps/artemis_cupps.dart';

import '../../classes/Requests/aea_request_req.dart';
import '../../classes/Requests/device_lock_req.dart';
import '../../classes/Requests/device_release_req.dart';
import '../../classes/Requests/device_status_req.dart';
import '../../classes/Requests/interface_levels_available_req.dart';
import '../../classes/Requests/interface_mode_req.dart';
import '../../classes/Requests/reader_read_request.dart';
import '../../classes/Responses/aea_response_res.dart';
import '../../classes/Responses/device_lock_res.dart';
import '../../classes/Responses/device_release_res.dart';
import '../../classes/Responses/device_status_res.dart';
import '../../classes/Responses/device_unlock_res.dart';
import '../../classes/Responses/reader_read_res.dart';
import '../../classes/notify.dart';
import '../../utility/cupps_util.dart';
import '../../utility/socket_util.dart';
import 'package:flutter/material.dart';
import 'package:xml/xml.dart';
import '../constants.dart';
import 'Devices/bc_device.dart';
import 'Devices/be_device.dart';
import 'Devices/bg_device.dart';
import 'Devices/bp_device.dart';
import 'Devices/bt_device.dart';
import 'Devices/dd_device.dart';
import 'Devices/ms_device.dart';
import 'Devices/oc_device.dart';
import 'Devices/pr_device.dart';
import 'Devices/zi_device.dart';
import 'Devices/zl_device.dart';
import 'Requests/device_acquire_req.dart';
import 'Requests/device_unlock_req.dart';
import 'Requests/interface_level_req.dart';
import 'Responses/bye_res.dart';
import 'Responses/device_acquire_res.dart';
import 'Responses/interface_level_res.dart';
import 'Responses/interface_levels_available_res.dart';
import 'Responses/interface_mode_res.dart';
import 'classes.dart';

class Device {
  SocketService? socketService;
  bool initializing = false;
  bool initialized = false;
  bool acquired = false;
  bool locked = false;
  bool modded = false;
  String statusMsg = "Not Initialized";
  List<Completer> _printQueue = [];
  final String deviceIndex;
  final String deviceName;
  final String deviceParameterType;
  final List<InterfaceMode> supportedInterfaceModes;
  final DeviceParameter deviceParameter;
  final List<Device> devices;

  Device({
    required this.deviceIndex,
    required this.deviceName,
    required this.deviceParameterType,
    required this.devices,
    required this.supportedInterfaceModes,
    required this.deviceParameter,
  });

  factory Device.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String deviceIndex = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "deviceIndex").value;
    String deviceName = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "deviceName").value;
    String deviceParameterType =
        document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "deviceParameterType").value;

    XmlElement? interfaceModesElement = document.firstChild!.getElement("supportedInterfaceModes");
    List<XmlElement> devicesElements = document.firstChild!.findElements("device").toList();
    List<XmlElement> interfacesElements = interfaceModesElement!.findElements("interfaceMode").toList();

    List<InterfaceMode> interfaceModes = interfacesElements.map((e) => InterfaceMode.fromXML(e.toXmlString())).toList();

    List<Device> devices = devicesElements.map((e) => Device.fromXML(e.toXmlString())).toList();
    DeviceParameter deviceParameter;
    XmlElement? deviceParameterElement = document.firstChild!.getElement(deviceParameterType);
    String deviceParameterXML = deviceParameterElement!.toXmlString();
    // print("$deviceParameterType $deviceIndex ${devices.map((e) => e.deviceIndex).toList()}");
    switch (deviceParameterType) {
      case "bpDeviceParameter":
        deviceParameter = BpDeviceParameter.fromXML(deviceParameterXML);
        BpDeviceParameter dp = BpDeviceParameter.fromXML(deviceParameterXML);
        BpDevice(
          deviceIndex: deviceIndex,
          deviceName: deviceName,
          deviceParameterType: deviceParameterType,
          devices: devices,
          supportedInterfaceModes: interfaceModes,
          deviceParameter: dp,
        );
        break;
      case "btDeviceParameter":
        deviceParameter = BtDeviceParameter.fromXML(deviceParameterXML);
        BtDeviceParameter dp = BtDeviceParameter.fromXML(deviceParameterXML);
        BtDevice(
          deviceIndex: deviceIndex,
          deviceName: deviceName,
          deviceParameterType: deviceParameterType,
          devices: devices,
          supportedInterfaceModes: interfaceModes,
          deviceParameter: dp,
        );
        break;
      case "bcDeviceParameter":
        deviceParameter = BcDeviceParameter.fromXML(deviceParameterXML);
        BcDeviceParameter dp = BcDeviceParameter.fromXML(deviceParameterXML);
        BcDevice(
          deviceIndex: deviceIndex,
          deviceName: deviceName,
          deviceParameterType: deviceParameterType,
          devices: devices,
          supportedInterfaceModes: interfaceModes,
          deviceParameter: dp,
        );
        break;
      case "beDeviceParameter":
        deviceParameter = BeDeviceParameter.fromXML(deviceParameterXML);
        BeDeviceParameter dp = BeDeviceParameter.fromXML(deviceParameterXML);
        BeDevice(
          deviceIndex: deviceIndex,
          deviceName: deviceName,
          deviceParameterType: deviceParameterType,
          devices: devices,
          supportedInterfaceModes: interfaceModes,
          deviceParameter: dp,
        );
        break;
      case "ocDeviceParameter":
        deviceParameter = OcDeviceParameter.fromXML(deviceParameterXML);
        OcDeviceParameter dp = OcDeviceParameter.fromXML(deviceParameterXML);
        OcDevice(
          deviceIndex: deviceIndex,
          deviceName: deviceName,
          deviceParameterType: deviceParameterType,
          devices: devices,
          supportedInterfaceModes: interfaceModes,
          deviceParameter: dp,
        );
        break;
      case "msDeviceParameter":
        deviceParameter = MsDeviceParameter.fromXML(deviceParameterXML);
        MsDeviceParameter dp = MsDeviceParameter.fromXML(deviceParameterXML);
        MsDevice(
          deviceIndex: deviceIndex,
          deviceName: deviceName,
          deviceParameterType: deviceParameterType,
          devices: devices,
          supportedInterfaceModes: interfaceModes,
          deviceParameter: dp,
        );
        break;
      case "prDeviceParameter":
        deviceParameter = PrDeviceParameter.fromXML(deviceParameterXML);
        PrDeviceParameter dp = PrDeviceParameter.fromXML(deviceParameterXML);
        PrDevice(
          deviceIndex: deviceIndex,
          deviceName: deviceName,
          deviceParameterType: deviceParameterType,
          devices: devices,
          supportedInterfaceModes: interfaceModes,
          deviceParameter: dp,
        );
        break;
      case "bgDeviceParameter":
        deviceParameter = BgDeviceParameter.fromXML(deviceParameterXML);
        BgDeviceParameter dp = BgDeviceParameter.fromXML(deviceParameterXML);
        BgDevice(
          deviceIndex: deviceIndex,
          deviceName: deviceName,
          deviceParameterType: deviceParameterType,
          devices: devices,
          supportedInterfaceModes: interfaceModes,
          deviceParameter: dp,
        );
        break;
      case "ddDeviceParameter":
        deviceParameter = DdDeviceParameter.fromXML(deviceParameterXML);
        DdDeviceParameter dp = DdDeviceParameter.fromXML(deviceParameterXML);
        DdDevice(
          deviceIndex: deviceIndex,
          deviceName: deviceName,
          deviceParameterType: deviceParameterType,
          devices: devices,
          supportedInterfaceModes: interfaceModes,
          deviceParameter: dp,
        );
        break;
      case "zlDeviceParameter":
        deviceParameter = ZlDeviceParameter.fromXML(deviceParameterXML);
        ZlDeviceParameter dp = ZlDeviceParameter.fromXML(deviceParameterXML);
        ZlDevice(
          deviceIndex: deviceIndex,
          deviceName: deviceName,
          deviceParameterType: deviceParameterType,
          devices: devices,
          supportedInterfaceModes: interfaceModes,
          deviceParameter: dp,
        );
        break;
      case "ziDeviceParameter":
        deviceParameter = ZiDeviceParameter.fromXML(deviceParameterXML);
        ZiDeviceParameter dp = ZiDeviceParameter.fromXML(deviceParameterXML);
        ZiDevice(
          deviceIndex: deviceIndex,
          deviceName: deviceName,
          deviceParameterType: deviceParameterType,
          devices: devices,
          supportedInterfaceModes: interfaceModes,
          deviceParameter: dp,
        );
        break;
      default:
        deviceParameter = BpDeviceParameter.fromXML(deviceParameterXML);
    }

    return Device(
        deviceIndex: deviceIndex,
        deviceName: deviceName,
        deviceParameterType: deviceParameterType,
        devices: devices,
        supportedInterfaceModes: interfaceModes,
        deviceParameter: deviceParameter);
  }

  String toXML() {
    return '''
    <device deviceIndex = "$deviceIndex" deviceName = "$deviceName" deviceParameterType = "$deviceParameterType" >
    <supportedInterfaceModes>
    ${supportedInterfaceModes.map((e) => e.toXML()).join()}
    </supportedInterfaceModes>
    ${deviceParameter.toXML()}
    ${devices.map((e) => e.toXML()).join()}
    </device>
    ''';
  }

  DeviceType get getType => DeviceType.values.firstWhere((element) => element.parameter == deviceParameterType);

  int get currentMessageID => socketService!.messageID;

  bool get isConnected => socketService != null && socketService!.isConnected;

  bool get isPrinting => _printQueue.isNotEmpty;

  bool get haveLock => [DeviceType.bc, DeviceType.oc, DeviceType.ms, DeviceType.pr].contains(getType);

  String get icon => !isConnected
      ? "${AssetImages.iconsAddress}/${getType.name}/${getType.name}_natural.png"
      : isPrinting
          ? "${AssetImages.iconsAddress}/${getType.name}/${getType.name}_printing.gif"
          : initializing
              ? "${AssetImages.iconsAddress}/${getType.name}/${getType.name}_in.png"
              : (getType == DeviceType.bg && statusMsg.contains("NotShareable"))
                  ? "${AssetImages.iconsAddress}/${getType.name}/${getType.name}_inuse.png"
                  : initialized
                      ? (deviceParameter.status.isReady && locked)
                          ? "${AssetImages.iconsAddress}/${getType.name}/${getType.name}_locked.png"
                          : deviceParameter.status.image
                      : "${AssetImages.iconsAddress}/${getType.name}/${getType.name}_natural.png";

  void disableRetry() {
    if (isConnected && socketService != null) {
      socketService!.disableRetry();
    }
  }

  void setConnectionChecker(
      {Duration timeout = const Duration(seconds: 12),
      Duration interval = const Duration(seconds: 45),
      required Function retryFunction}) {
    socketService!.setConnectionChecker(timeout: timeout, interval: interval, retryFunction: retryFunction);
  }

  onDeviceDisconnect() {
    log("Device $deviceIndex : $deviceName Disconnected");
    socketService = null;
    locked = false;
    acquired = false;
    statusMsg = "Disconnected";
    // CUPPS.removeDevice(deviceIndex);
    CUPPS().notifier!();
  }

  Future<String> send({required String xml}) async {
    return socketService!.sendSocketMessage(xml);
  }

  Future<void> respond({required String xml}) async {
    return socketService!.respond(xml);
  }

  Future<CuppsCommandResponse> connect() async {
    try {
      if (!isConnected) {
        String ip = deviceParameter.ipAndPort.ip;
        int port = int.tryParse(deviceParameter.ipAndPort.port) ?? 0;
        log("creating device socket service");
        socketService = SocketService();
        await socketService!.initialize(
          onLost: () {},
          ip: ip,
          port: port,
          rh: _responseHandler,
          onClose: onDeviceDisconnect,
          logSuff: getType.name,
          logID: "Device $deviceName",
        );
        CUPPS().notifier!();
        return CuppsCommandResponse(msg: "Device :$deviceName Connected Successfully!", status: true);
      } else {
        return CuppsCommandResponse(msg: "Device :$deviceName Already Connected!", status: true);
      }
    } catch (e) {
      return CuppsCommandResponse(msg: "Device $deviceName connect Exception: $e", status: false);
    }
  }

  Future<CuppsCommandResponse> initialize() async {
    try {
      initializing = true;
      statusMsg = "Initializing...";
      CUPPS().notifier!();

      InterfaceLevelsAvailableRequest req =
          InterfaceLevelsAvailableRequest(messageID: currentMessageID, hsXsdVersion: '01.01.0128');
      String interfaceLevelsAvailableResXML = await send(xml: req.toXML());
      InterfaceLevelsAvailableResponse interfaceLevelsAvailableRes =
          InterfaceLevelsAvailableResponse.fromXML(interfaceLevelsAvailableResXML);
      if (interfaceLevelsAvailableRes.result.toLowerCase() == "ok") {
        InterfaceLevelRequest req = InterfaceLevelRequest(
            level: interfaceLevelsAvailableRes.interfaceLevels.firstWhere((element) => element.level == "01.03").level,
            messageID: currentMessageID);
        String interfaceLevelResXML = await send(xml: req.toXML());
        InterfaceLevelResponse interfaceLevelRes = InterfaceLevelResponse.fromXML(interfaceLevelResXML);
        if (interfaceLevelRes.result.toLowerCase() == "ok") {
          CuppsCommandResponse acquireRes = await acquire();
          if (acquireRes.status) {
            CuppsCommandResponse modeRes = await mode();
            // return CuppsCommandResponse(
            //     status: modeRes.status,
            //     msg: "Device: $deviceName Initialized Successfully");
            if (modeRes.status) {
              setConnectionChecker(retryFunction: restart);
              CuppsCommandResponse refreshStatusRes = await refreshStatus();
              initializing = false;
              modded = true;
              initialized = true;
              statusMsg = "Initialized";
              CUPPS().notifier!();
              return CuppsCommandResponse(status: refreshStatusRes.status, msg: "Device: $deviceName Initialized Successfully");
            } else {
              initializing = false;
              modded = false;
              statusMsg = modeRes.msg;
              CUPPS().notifier!();
              return modeRes;
            }
          } else {
            initializing = false;
            statusMsg = acquireRes.msg;
            CUPPS().notifier!();
            return acquireRes;
          }
        } else {
          initializing = false;
          statusMsg = interfaceLevelRes.result;
          CUPPS().notifier!();
          return CuppsCommandResponse(msg: interfaceLevelRes.result, status: false);
        }
      } else {
        initializing = false;
        statusMsg = interfaceLevelsAvailableRes.result;
        CUPPS().notifier!();
        return CuppsCommandResponse(msg: interfaceLevelsAvailableRes.result, status: false);
      }
    } catch (e) {
      initializing = false;
      statusMsg = "Initialize Exception";
      CUPPS().notifier!();
      return CuppsCommandResponse(msg: "Device $deviceName initialize Exception: $e", status: false);
    }
  }

  Future<CuppsCommandResponse> restart() async {
    // try {
    final releaseRes = await release();
    if (releaseRes.status) {
      final connectRes = await connect();
      if (connectRes.status) {
        final initRes = await initialize();
        return initRes;
      } else {
        return connectRes;
      }
    } else {
      return releaseRes;
    }
    // } catch (e) {
    //   initializing = false;
    //   statusMsg = "Initialize Exception";
    //   CUPPS().notifier!();
    //   return CuppsCommandResponse(msg: "Device $deviceName restart Exception: $e", status: false);
    // }
  }

  Future<CuppsCommandResponse> interfaceLevelsAvailable() async {
    InterfaceLevelsAvailableRequest req = InterfaceLevelsAvailableRequest(messageID: currentMessageID, hsXsdVersion: '01.03');
    String interfaceLevelsAvailableResXML = await send(xml: req.toXML());
    InterfaceLevelsAvailableResponse interfaceLevelsAvailableRes =
        InterfaceLevelsAvailableResponse.fromXML(interfaceLevelsAvailableResXML);
    if (interfaceLevelsAvailableRes.result.toLowerCase() == "ok") {
      return CuppsCommandResponse(status: true, msg: "Available Interface Levels Received");
    } else {
      return CuppsCommandResponse(status: false, msg: interfaceLevelsAvailableRes.result);
    }
  }

  Future<CuppsCommandResponse> interfaceLevel(String level) async {
    InterfaceLevelRequest req = InterfaceLevelRequest(level: level, messageID: currentMessageID);
    String interfaceLevelResXML = await send(xml: req.toXML());
    InterfaceLevelResponse interfaceLevelRes = InterfaceLevelResponse.fromXML(interfaceLevelResXML);
    if (interfaceLevelRes.result.toLowerCase() == "ok") {
      return CuppsCommandResponse(status: true, msg: "Interface Level Set Successfully");
    } else {
      return CuppsCommandResponse(status: false, msg: interfaceLevelRes.result);
    }
  }

  Future<CuppsCommandResponse> acquire() async {
    String deviceToken = CUPPS().deviceToken!;
    String airline = CUPPS().airline;
    DeviceAcquireRequest req = DeviceAcquireRequest(
        deviceName: deviceName, deviceToken: deviceToken, airlineID: airline, msFoidMasking: "", messageID: currentMessageID);
    String acquireResXML = await send(xml: req.toXML());
    DeviceAcquireResponse acquireRes = DeviceAcquireResponse.fromXML(acquireResXML);
    if (acquireRes.result.toLowerCase() == "ok") {
      acquired = true;
      CUPPS().notifier!();

      return CuppsCommandResponse(status: true, msg: "Device :$deviceName Successfully Acquired");
    } else {
      return CuppsCommandResponse(status: false, msg: acquireRes.result);
    }
  }

  Future<CuppsCommandResponse> mode() async {
    String mode = supportedInterfaceModes.first.mode;
    if (getType == DeviceType.bg) {
      mode = "aea";
    }
    InterfaceModeRequest req = InterfaceModeRequest(mode: mode, messageID: currentMessageID);
    String modeResXML = await send(xml: req.toXML());
    InterfaceModeResponse modeRes = InterfaceModeResponse.fromXML(modeResXML);
    if (modeRes.result.toLowerCase() == "ok") {
      statusMsg = "Initialized";
      acquired = true;
      modded  =true;
      CUPPS().notifier!();
      return CuppsCommandResponse(status: true, msg: "Device :$deviceName Interface Mode is Set:$mode");
    } else {
      acquired = false;
      modded  =false;
      CUPPS().notifier!();
      if(getType == DeviceType.bg){
        Future.delayed(const Duration(seconds: 45),(){
          restart();
        });
      }
      return CuppsCommandResponse(status: false, msg: modeRes.result);
    }
  }

  Future<CuppsCommandResponse> refreshStatus() async {
    try {
      DeviceStatusRequest req = DeviceStatusRequest(messageID: currentMessageID);
      String deviceStatusResXML = await send(xml: req.toXML());
      DeviceStatusResponse deviceStatusRes = DeviceStatusResponse.fromXML(deviceStatusResXML);
      // print("(TO DEVICE $deviceIndex) Received DeviceStatus ${deviceStatusRes.result}");
      if (deviceStatusRes.result.toLowerCase() == "ok" || deviceStatusRes.result.toLowerCase() == "") {
        // print(deviceStatusRes.status.map((e) => e.toXML()).toList());
        if (deviceStatusRes.status.where((element) => element.toXML().contains("${getType.name}Status")).isNotEmpty) {
          // print("Status DETECTED");
          deviceParameter.status =
              deviceStatusRes.status.firstWhere((element) => element.toXML().contains("${getType.name}Status"));
          CUPPS().notifier!();
          return CuppsCommandResponse(status: true, msg: "Device :$deviceName Status Refreshed");
        } else {
          CUPPS().notifier!();
          return CuppsCommandResponse(status: false, msg: "Device :$deviceName Status Not Found in Response");
        }
      } else {
        return CuppsCommandResponse(status: false, msg: deviceStatusRes.result);
      }
    } catch (e) {
      return CuppsCommandResponse(status: false, msg: "Device $deviceName refresh Exception: $e");
    }
  }

  Future<CuppsCommandResponse> lock({bool renew = false}) async {
    if (haveLock) {
      try {
        DeviceLockRequest req =
            DeviceLockRequest(messageID: currentMessageID, lockMethod: 'byConnection', renew: renew ? "true" : "false");
        String lockResXML = await send(xml: req.toXML());
        DeviceLockResponse lockRes = DeviceLockResponse.fromXML(lockResXML);
        if (lockRes.result.toLowerCase() == "ok") {
          locked = true;
          CUPPS().notifier!();
          return CuppsCommandResponse(status: true, msg: "Device :$deviceName Locked Successfully");
        } else {
          return CuppsCommandResponse(status: false, msg: lockRes.result);
        }
      } catch (e) {
        return CuppsCommandResponse(status: false, msg: "Device $deviceName lock Exception: $e");
      }
    } else {
      return CuppsCommandResponse(status: false, msg: "Illogical Action");
    }
  }

  Future<CuppsCommandResponse> unlock() async {
    if (haveLock) {
      try {
        if (locked) {
          DeviceUnlockRequest req = DeviceUnlockRequest(messageID: currentMessageID);
          String unlockResXML = await send(xml: req.toXML());
          DeviceUnlockResponse unlockRes = DeviceUnlockResponse.fromXML(unlockResXML);
          if (unlockRes.result.toLowerCase() == "ok") {
            locked = false;
            CUPPS().notifier!();
            return CuppsCommandResponse(status: true, msg: "Device :$deviceName Unlocked Successfully");
          } else {
            if (unlockRes.result == "deviceNotLocked") {
              locked = false;
              CUPPS().notifier!();
              return CuppsCommandResponse(status: true, msg: unlockRes.result);
            } else {
              return CuppsCommandResponse(status: false, msg: unlockRes.result);
            }
          }
        } else {
          return CuppsCommandResponse(status: false, msg: "Device Not Locked");
        }
      } catch (e) {
        return CuppsCommandResponse(status: false, msg: "Device $deviceName unlock Exception: $e");
      }
    } else {
      return CuppsCommandResponse(status: false, msg: "Illogical Action");
    }
  }

  Future<CuppsCommandResponse> release() async {
    try {
      if (acquired && isConnected) {
        if (locked) {
          DeviceUnlockRequest req = DeviceUnlockRequest(messageID: currentMessageID);
          String unlockResXML = await send(xml: req.toXML());
          DeviceUnlockResponse unlockRes = DeviceUnlockResponse.fromXML(unlockResXML);
          if (unlockRes.result.toLowerCase() == "ok") {
            locked = false;
            DeviceReleaseRequest req = DeviceReleaseRequest(messageID: currentMessageID);
            String deviceReleaseResXML = await send(xml: req.toXML());
            DeviceReleaseResponse deviceReleaseRes = DeviceReleaseResponse.fromXML(deviceReleaseResXML);
            if (deviceReleaseRes.result.toLowerCase() == "ok") {
              socketService!.disableRetry();
              await socketService!.disconnect();
              return CuppsCommandResponse(status: true, msg: "Device:$deviceName Released Successfully");
            } else {
              return CuppsCommandResponse(status: false, msg: deviceReleaseRes.result);
            }
          } else {
            socketService!.disableRetry();
            await socketService!.disconnect();
            return CuppsCommandResponse(status: true, msg: unlockRes.result);
          }
        } else {
          log("Not Locked");
          DeviceReleaseRequest req = DeviceReleaseRequest(messageID: currentMessageID);
          String deviceReleaseResXML = await send(xml: req.toXML());
          log(deviceReleaseResXML);
          DeviceReleaseResponse deviceReleaseRes = DeviceReleaseResponse.fromXML(deviceReleaseResXML);
          if (deviceReleaseRes.result.toLowerCase() == "ok") {
            socketService!.disableRetry();
            await socketService!.disconnect();
            return CuppsCommandResponse(status: true, msg: "Device :$deviceName Released Successfully");
          } else {
            socketService!.disableRetry();
            await socketService!.disconnect();
            return CuppsCommandResponse(status: true, msg: deviceReleaseRes.result);
          }
        }
      } else {
        if (socketService != null) {
          socketService!.disableRetry();
        }
        return CuppsCommandResponse(status: true, msg: "Device :$deviceName is not Acquired");
      }
    } catch (e) {
      if (socketService != null) {
        socketService!.disableRetry();
      }
      return CuppsCommandResponse(status: false, msg: "Device $deviceName release Exception: $e");
    }
  }

  Future<CuppsCommandResponse> aeaRequest(String text) async {
    try {
      AeaRequestRequest req = AeaRequestRequest(messageID: currentMessageID, texts: [text], binaries: []);
      String aeaResXML = await send(xml: req.toXML());
      AeaResponseResponse aeaResponseResponse = AeaResponseResponse.fromXML(aeaResXML);
      if (aeaResponseResponse.result.toLowerCase() == "ok") {
        return CuppsCommandResponse(status: true, msg: "Aea Request Sent Successfully to Device: :$deviceName");
      } else {
        return CuppsCommandResponse(status: false, msg: aeaResponseResponse.result);
      }
    } catch (e) {
      return CuppsCommandResponse(status: false, msg: "Device $deviceName aeaRequest Exception: $e");
    }
  }

  Future<CuppsCommandResponse> _handleBgResponse(BgResponse res,
      {int successBeepDuration = 5, int failureBeepDuration = 2}) async {
    try {
      if (res.status) {
        AeaRequestRequest displayReq = AeaRequestRequest(messageID: currentMessageID, texts: ["MG#B#A${res.message}"]);
        send(xml: displayReq.toXML());
        AeaRequestRequest successBeepReq = AeaRequestRequest(messageID: currentMessageID, texts: ["CC#0$successBeepDuration"]);
        String beepResXML = await send(xml: successBeepReq.toXML());
        AeaResponseResponse beepRes = AeaResponseResponse.fromXML(beepResXML);

        // AeaRequestRequest displayReq = AeaRequestRequest(messageID: currentMessageID, texts: ["MG#B#A${res.message}"]);
        // String displayResXML = await send(xml: displayReq.toXML());
        // AeaResponseResponse displayRes = AeaResponseResponse.fromXML(displayResXML);



        if (beepRes.result.toLowerCase() == "ok") {
          return CuppsCommandResponse(status: true, msg: "BG Response Handled Successfully");
        } else {
          return CuppsCommandResponse(status: false, msg: "Display Result: ${beepRes.result}");
        }
      } else {
        AeaRequestRequest displayReq = AeaRequestRequest(messageID: currentMessageID, texts: ["MG#B#A${res.message}"]);
        send(xml: displayReq.toXML());
        AeaRequestRequest failureBeepReq = AeaRequestRequest(messageID: currentMessageID, texts: ["CB#0$failureBeepDuration"]);
        String beepResXML = await send(xml: failureBeepReq.toXML());
        AeaResponseResponse beepRes = AeaResponseResponse.fromXML(beepResXML);

        // AeaRequestRequest displayReq = AeaRequestRequest(messageID: currentMessageID, texts: ["MG#B#A${res.message}"]);
        // String displayResXML = await send(xml: displayReq.toXML());
        // AeaResponseResponse displayRes = AeaResponseResponse.fromXML(displayResXML);



        if (beepRes.result.toLowerCase() == "ok") {
          return CuppsCommandResponse(status: true, msg: "BG Response Handled Successfully");
        } else {
          return CuppsCommandResponse(status: false, msg: "Display Result:${beepRes.result}");
        }
      }
    } catch (e) {
      return CuppsCommandResponse(status: false, msg: "Device $deviceName response Exception: $e");
    }
  }

  Future<CuppsCommandResponse> print(String text) async {
    final completer = Completer<CuppsCommandResponse>();

    final reqRes = await aeaRequest(text);
    if (reqRes.status) {
      _printQueue.add(completer);
      CUPPS().notifier!();
      log("isPrinting $isPrinting");
      return completer.future.timeout(const Duration(seconds: 12), onTimeout: () {
        _printQueue.remove(completer);
        CUPPS().notifier!();
        return CuppsCommandResponse(status: false, msg: "Print Timeout");
      });
    } else {
      return reqRes;
    }
  }

  _responseHandler(CuppsBase response, String rawXML) async {
    switch (response.messageName) {
      // case CuppsMessageID.interfaceLevelsAvailable:
      //   {
      //     InterfaceLevelsAvailableResponse res = InterfaceLevelsAvailableResponse.fromXML(rawXML);
      //     InterfaceLevelRequest req = InterfaceLevelRequest(level: res.interfaceLevels.first.level, messageID: currentMessageID);
      //     print("(TO DEVICE $deviceIndex) Received InterfaceLevelsAvailable");
      //     send(xml: req.toXML());
      //     break;
      //   }
      // case CuppsMessageID.interfaceLevel:
      //   {
      //     InterfaceLevelResponse res = InterfaceLevelResponse.fromXML(rawXML);
      //     print("(TO DEVICE $deviceIndex) Received InterfaceLevel");
      //     String deviceToken = CUPPS().deviceToken!;
      //     String airline = CUPPS().airline;
      //     DeviceAcquireRequest req = DeviceAcquireRequest(
      //         deviceName: deviceName,
      //         deviceToken: deviceToken,
      //         airlineID: airline,
      //         msFoidMasking: "",
      //         messageID: currentMessageID);
      //     send(xml: req.toXML());
      //     break;
      //   }
      // case CuppsMessageID.deviceAcquire:
      //   {
      //     DeviceAcquireResponse res = DeviceAcquireResponse.fromXML(rawXML);
      //     if (res.result.toLowerCase() != "ok" || res.device == null) {
      //       print("(TO DEVICE $deviceIndex) DEVICE NOT ACQUIRED ${res.result}");
      //     } else {
      //       acquired = true;
      //       print("(TO DEVICE $deviceIndex) DEVICE ACQUIRED ${res.device!.deviceName}");
      //       print("availableModes : ${supportedInterfaceModes.map((e) => e.mode).toList()}");
      //       String mode = supportedInterfaceModes.first.mode;
      //       if (getType == DeviceType.BG) {
      //         mode = "aea";
      //       }
      //       InterfaceModeRequest req = InterfaceModeRequest(mode: mode, messageID: currentMessageID);
      //       send(xml: req.toXML());
      //     }
      //
      //     break;
      //   }
      // case CuppsMessageID.deviceReleaseResponse:
      //   {
      //     DeviceReleaseResponse res = DeviceReleaseResponse.fromXML(rawXML);
      //     if (res.result.toLowerCase() == "ok") {
      //       acquired = false;
      //       // disconnect();
      //     }
      //     break;
      //   }
      // case CuppsMessageID.interfaceMode:
      //   {
      //     InterfaceModeResponse res = InterfaceModeResponse.fromXML(rawXML);
      //     print("(TO DEVICE $deviceIndex) Received InterfaceMode ${res.result}");
      //     // AeaRequestRequest req = AeaRequestRequest(text: XMLExamples.aeaRequest2);
      //     DeviceStatusRequest req = DeviceStatusRequest(messageID: currentMessageID);
      //     send(xml: req.toXML());
      //     break;
      //   }
      // case CuppsMessageID.deviceStatus:
      //   {
      //     DeviceStatusResponse res = DeviceStatusResponse.fromXML(rawXML);
      //     print("(TO DEVICE $deviceIndex) Received DeviceStatus ${res.result}");
      //     print(res.status.map((e) => e.toXML()).toList());
      //     if (res.status.where((element) => element.toXML().contains("${getType.name}Status")).isNotEmpty) {
      //       print("Status DETECTED");
      //       deviceParameter.status = res.status.firstWhere((element) => element.toXML().contains("${getType.name}Status"));
      //     }
      //
      //     // AeaRequestRequest req = AeaRequestRequest(text: XMLExamples.aeaRequest2);
      //     // DeviceStatusRequest req = DeviceStatusRequest();
      //     // send(xml: req.toXML());
      //
      //     break;
      //   }
      // case CuppsMessageID.aeaResponse:
      //   {
      //     AeaResponseResponse res = AeaResponseResponse.fromXML(rawXML);
      //     print("(TO DEVICE $deviceIndex) Received AeaResponse :${res.result}");
      //
      //     break;
      //   }
      case CuppsMessageID.aeaRequest:
        {
          AeaRequestRequest deviceReq = AeaRequestRequest.fromXML(rawXML);
          log("(TO DEVICE $deviceIndex) Received AeaRequest :${deviceReq.texts.first}");

          AeaResponseResponse req = AeaResponseResponse(messageID: deviceReq.messageID, messageName: "aeaResponse", result: "OK");
          respond(xml: req.toXML());

          ///Aea Request from Bg devices could be a read data => should validate and response
          if (getType == DeviceType.bg) {
            String aeaRequestText = deviceReq.texts.first;
            log("Device is BG ,Maybe its a Read Data");
            if (aeaRequestText.contains("BCRI")) {
              ///example &lt;![CDATA[HDCBCRI#6M1JACOB ROBERT        QLGGY   YYZADBTK 2525 311Y9C  37   100&lt;]]&gt;
              log("Its Read Data from BG Device Should process Data");
              String barcodeInfo = aeaRequestText.split("M1").first;
              String rawData = aeaRequestText.replaceFirst(barcodeInfo, "");
              // log("RawData : $rawData");
              BgData bgData = BgData(value: rawData);
              if (CUPPS().bgListener != null) {
                BgResponse bgRes = await CUPPS().bgListener!(bgData);
                final bgHandlingResponse = await _handleBgResponse(bgRes);
                log("$bgHandlingResponse");
              } else {
                log("BG Data Received But no Listener is Set");
                BgResponse bgRes = BgResponse(status: false, message: "Couldn't handle received data: No bgHandler is set");
                final bgHandlingResponse = await _handleBgResponse(bgRes);
                log("$bgHandlingResponse");
              }
            }
          } else if (getType == DeviceType.bp || getType == DeviceType.bt) {
            if (_printQueue.isNotEmpty) {
              String aeaRequestText = deviceReq.texts.first;
              if (aeaRequestText.contains("PROK") && !aeaRequestText.contains("PRERR")) {
                _printQueue.first.complete(CuppsCommandResponse(status: true, msg: "Print OK"));
                _printQueue.removeAt(0);
                log("isPrinting $isPrinting");
                CUPPS().notifier!();
              } else if (!aeaRequestText.contains("PROK") && aeaRequestText.contains("ERR")) {
                _printQueue.first.complete(CuppsCommandResponse(status: false, msg: "Print Error"));
                _printQueue.removeAt(0);
                log("isPrinting $isPrinting");
                CUPPS().notifier!();
              }
            }
          }
          break;
        }
      // case CuppsMessageID.applicationStopCommand:
      //   {
      //     ApplicationStopCommandResponse res = ApplicationStopCommandResponse.fromXML(rawXML);
      //     print("(TO DEVICE $deviceIndex) STOPPED");
      //     // print(res);
      //     break;
      //   }
      // case CuppsMessageID.deviceLockResponse:
      //   {
      //     DeviceLockResponse res = DeviceLockResponse.fromXML(rawXML);
      //     print("(TO DEVICE $deviceIndex) Device Lock:${res.result}");
      //     if (res.result.toLowerCase() == "ok") {
      //       locked = true;
      //     }
      //     // print(res);
      //     break;
      //   }
      // case CuppsMessageID.deviceUnlockResponse:
      //   {
      //     DeviceLockResponse res = DeviceLockResponse.fromXML(rawXML);
      //     print("(TO DEVICE $deviceIndex) Device Unlock:${res.result}");
      //     if (res.result.toLowerCase() == "ok") {
      //       locked = false;
      //     }
      //     // print(res);
      //     break;
      //   }
      // case CuppsMessageID.readerReadResponse:
      //   {
      //     ReaderReadResponse res = ReaderReadResponse.fromXML(rawXML);
      //     print("(TO DEVICE $deviceIndex) Reader Red: Data Count: ${res.dataList.length}");
      //     print("(TO DEVICE $deviceIndex) Reader Red:${res.result}");
      //     print(res.dataList.map((e) => e.data.toXML()).toList());
      //     if (getType == DeviceType.BC) {
      //       List<BcData> bc = res.dataList.map((e) => BcData.fromXML(e.data.toXML())).toList();
      //       print(bc.map((e) => e.dataSTR ?? "NULL").toList());
      //       CUPPS().bcListener(bc);
      //     } else if (getType == DeviceType.BG) {
      //       CUPPS().bgListener(res);
      //     } else if (getType == DeviceType.MS) {
      //       List<MsData> ms = res.dataList.map((e) => MsData.fromXML(e.data.toXML())).toList();
      //       CUPPS().msListener(ms);
      //     } else if (getType == DeviceType.OC) {
      //       List<OcData> oc = res.dataList.map((e) => OcData.fromXML(e.data.toXML())).toList();
      //       CUPPS().ocListener(oc);
      //     }
      //
      //     // print(res);
      //     break;
      //   }
      case CuppsMessageID.notify:
        {
          Notify res = Notify.fromXML(rawXML);
          switch (res.notificationType) {
            case NotificationType.dataAvailable:
              {
                ReaderReadRequest req = ReaderReadRequest(messageID: currentMessageID);
                String readResponseXML = await send(xml: req.toXML());
                ReaderReadResponse readRes = ReaderReadResponse.fromXML(readResponseXML);
                log("(TO DEVICE $deviceIndex) Reader Red: Data Count: ${readRes.dataList.length}");
                if (getType == DeviceType.bc) {
                  List<BcData> bc = readRes.dataList.map((e) => BcData.fromXML(e.data.toXML())).toList();
                  if (CUPPS().bcListener != null) {
                    CUPPS().bcListener!(bc);
                  } else {
                    log("BC Data Received But no Listener is Set");
                  }
                } else if (getType == DeviceType.ms) {
                  List<MsData> ms = readRes.dataList.map((e) => MsData.fromXML(e.data.toXML())).toList();
                  if (CUPPS().msListener != null) {
                    CUPPS().msListener!(ms);
                  } else {
                    log("MS Data Received But no Listener is Set");
                  }
                } else if (getType == DeviceType.oc) {
                  List<OcData> oc = readRes.dataList.map((e) => OcData.fromXML(e.data.toXML())).toList();
                  if (CUPPS().ocListener != null) {
                    CUPPS().ocListener!(oc);
                  } else {
                    log("OC Data Received But no Listener is Set");
                  }
                }

                break;
              }
            case NotificationType.deviceLockExpire:
              {
                // DeviceLockRequest req =
                //     DeviceLockRequest(messageID: currentMessageID, lockMethod: "byConnection", renew: "false");
                // send(xml: req.toXML());
                log("Should Renew Lock");
                lock(renew: true);
                break;
              }
            case NotificationType.deviceLockedExit:
              {
                locked = false;
                CUPPS().notifier!();
                break;
              }
            case NotificationType.deviceStatus:
              {
                // DeviceLockRequest req = DeviceLockRequest(messageID: currentMessageID, lockMethod: "byConnection",renew: "false");
                // send(xml: req.toXML());
                log("RECEIVED Device Status Notification");
                DeviceStatusNotification statusNotif = DeviceStatusNotification.fromXML(rawXML);
                XmlDocument doc = XmlDocument.parse(rawXML);
                XmlElement resElement = doc.rootElement.findAllElements("deviceStatusNotification").first;
                XmlElement statusElement =
                    resElement.childElements.where((element) => element.name.toString().endsWith("Status")).first;
                // deviceParameter.status = statusNotif.status;
                switch (getType) {
                  case DeviceType.bp:
                    BpStatus status = BpStatus.fromXML(statusElement.toXmlString());
                    deviceParameter.status = status;
                    break;
                  case DeviceType.bt:
                    BtStatus status = BtStatus.fromXML(statusElement.toXmlString());
                    deviceParameter.status = status;
                    break;
                  case DeviceType.bc:
                    BcStatus status = BcStatus.fromXML(statusElement.toXmlString());
                    deviceParameter.status = status;
                    break;
                  case DeviceType.be:
                    BeStatus status = BeStatus.fromXML(statusElement.toXmlString());
                    deviceParameter.status = status;
                    break;
                  case DeviceType.oc:
                    OcStatus status = OcStatus.fromXML(statusElement.toXmlString());
                    deviceParameter.status = status;
                    break;
                  case DeviceType.ms:
                    MsStatus status = MsStatus.fromXML(statusElement.toXmlString());
                    deviceParameter.status = status;
                    break;
                  case DeviceType.pr:
                    PrStatus status = PrStatus.fromXML(statusElement.toXmlString());
                    deviceParameter.status = status;
                    break;
                  case DeviceType.bg:
                    BgStatus status = BgStatus.fromXML(statusElement.toXmlString());
                    deviceParameter.status = status;
                    break;
                  case DeviceType.dd:
                    DdStatus status = DdStatus.fromXML(statusElement.toXmlString());
                    deviceParameter.status = status;
                    break;
                  case DeviceType.zl:
                    ZlStatus status = ZlStatus.fromXML(statusElement.toXmlString());
                    deviceParameter.status = status;
                    break;
                  case DeviceType.zi:
                    ZiStatus status = ZiStatus.fromXML(statusElement.toXmlString());
                    deviceParameter.status = status;
                    break;
                }

                // refreshStatus();

                // DeviceStatusNotification statusNotif = DeviceStatusNotification.fromXML(res.toXML());
                // if (statusNotif.status.toXML().contains("${getType.name}Status")) {
                //   // print("Status DETECTED");
                //   deviceParameter.status = Status.fromXML(statusNotif.status.toXML());
                //   CUPPS().notifier!();
                //   return CuppsCommandResponse(status: true, msg: "Device :$deviceName Status Refreshed");
                // }
                break;
              }
            default:
              {
                log("${res.notificationType.name} Notification Received on Device :$deviceName");
              }
          }

          break;
        }
      case CuppsMessageID.bye:
        {
          ByeResponse res = ByeResponse.fromXML(rawXML);
          log("(TO DEVICE $deviceIndex) GOODBYE");
          log(res.toXML());
          break;
        }
      default:
        {
          log(rawXML);
        }
    }
    CUPPS().notifier!();
  }
}

class InterfaceMode {
  final String mode;

  InterfaceMode({required this.mode});

  factory InterfaceMode.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String mode = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "mode").value;
    return InterfaceMode(mode: mode);
  }

  String toXML() {
    return '''<interfaceMode mode="$mode"/>''';
  }
}

class DeviceParameter {
  final VendorModelInfo vendorModelInfo;
  final IpAndPort ipAndPort;
  Status status;

  DeviceParameter({required this.vendorModelInfo, required this.ipAndPort, required this.status});

  String toXML() {
    return '''''';
  }
}

class VendorModelInfo {
  final String miscInfo;
  final String model;
  final String vendor;
  final String? poCapabilities;

  VendorModelInfo({required this.miscInfo, required this.model, required this.vendor, this.poCapabilities});

  factory VendorModelInfo.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String miscInfo =
        document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "miscInfo", orElse: orElseEmpty).value;
    String model =
        document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "model", orElse: orElseEmpty).value;
    String vendor =
        document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "vendor", orElse: orElseEmpty).value;
    String poCapabilities = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "poCapabilities", orElse: orElseEmpty)
        .value;
    return VendorModelInfo(
        miscInfo: miscInfo, model: model, vendor: vendor, poCapabilities: poCapabilities.isEmpty ? null : poCapabilities);
  }

  String toXML() {
    return '''<vendorModelInfo miscInfo="$miscInfo" model="$model" vendor="$vendor" ${poCapabilities == null ? "" : 'poCapabilities="$poCapabilities"'}/>''';
  }
}

class IpAndPort {
  final String hostName;
  final String ip;
  final String port;

  IpAndPort({required this.hostName, required this.ip, required this.port});

  factory IpAndPort.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String hostName = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "hostName").value;
    String ip = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "ip").value;
    String port = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "port").value;
    return IpAndPort(hostName: hostName, ip: ip, port: port);
  }

  String toXML() {
    return '''<ipAndPort hostName="$hostName" ip="$ip" port="$port"/>''';
  }

  @override
  String toString() {
    return "$ip:$port";
  }
}

class SupportedStock {
  final String stockName;
  final String supportsColor;
  final String stockWidth;
  final String stockHeight;
  final String leftMargin;
  final String rightMargin;
  final String topMargin;
  final String bottomMargin;
  final String perforationLeft;

  SupportedStock(
      {required this.stockName,
      required this.supportsColor,
      required this.stockWidth,
      required this.stockHeight,
      required this.leftMargin,
      required this.rightMargin,
      required this.topMargin,
      required this.bottomMargin,
      required this.perforationLeft});

  factory SupportedStock.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String stockName = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "stockName", orElse: orElseEmpty)
        .value;
    String supportsColor = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "supportsColor", orElse: orElseEmpty)
        .value;
    String stockWidth = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "stockWidth").value;
    String stockHeight = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "stockHeight").value;
    String leftMargin = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "leftMargin").value;
    String rightMargin = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "rightMargin").value;
    String topMargin = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "topMargin").value;
    String bottomMargin =
        document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "bottomMargin").value;
    String perforationLeft =
        document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "perforationLeft").value;
    return SupportedStock(
      stockName: stockName,
      supportsColor: supportsColor,
      stockWidth: stockWidth,
      stockHeight: stockHeight,
      leftMargin: leftMargin,
      rightMargin: rightMargin,
      topMargin: topMargin,
      bottomMargin: bottomMargin,
      perforationLeft: perforationLeft,
    );
  }

  String toXML() {
    return '''<supportedStock stockName="$stockName" supportsColor="$supportsColor" stockWidth="$stockWidth" stockHeight="$stockHeight" leftMargin="$leftMargin" rightMargin="$rightMargin" topMargin="$topMargin" bottomMargin="$bottomMargin" perforationLeft="$perforationLeft"/>''';
  }
}

class Status {
  final String desc;
  final String init;
  final String powerOff;
  final String ready;
  final String unknown;

  Status({required this.desc, required this.init, required this.powerOff, required this.ready, required this.unknown});

  factory Status.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String typeCode = document.firstElementChild!.name.toString().substring(0, 2);
    switch (typeCode) {
      case "bp":
        return BpStatus.fromXML(xmlSTR);
      case "bt":
        return BtStatus.fromXML(xmlSTR);
      case "bg":
        return BgStatus.fromXML(xmlSTR);
      case "bc":
        return BcStatus.fromXML(xmlSTR);
      case "be":
        return BeStatus.fromXML(xmlSTR);
      case "dd":
        return DdStatus.fromXML(xmlSTR);
      case "zl":
        return ZlStatus.fromXML(xmlSTR);
      case "zi":
        return ZiStatus.fromXML(xmlSTR);
      case "pr":
        return PrStatus.fromXML(xmlSTR);
      case "oc":
        return OcStatus.fromXML(xmlSTR);
      case "ms":
        return MsStatus.fromXML(xmlSTR);
    }

    String desc =
        document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "desc", orElse: orElseEmpty).value;
    String init = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "init").value;
    String powerOff = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "powerOff").value;
    String ready = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "ready").value;
    String unknown = document.firstChild!.attributes.firstWhere((element) => element.name.toString() == "unknown").value;
    return BcStatus(desc: desc, init: init, powerOff: powerOff, ready: ready, unknown: unknown);
  }

  String toXML() {
    return '''<status desc="$desc" init="$init" ready="$ready"/>''';
  }

  bool get isInit => init == "true";

  bool get isReady => ready == "true";

  bool get isOff => powerOff == "true";

  Color get color => isReady
      ? Colors.green
      : isOff
          ? Colors.red
          : Colors.grey;

  @override
  String toString() {
    if (isOff) return "Disconnect";
    if (isLocked) return "Locked";
    if (isInit) return "Initializing";
    if (isReady) return "Ready";
    return "$init $powerOff $ready $unknown";
  }

  bool get isLocked => unknown == "locked" || desc == "locked";

  String get image {
    return AssetImages.unknownDevice;
  }
}
