import 'package:xml/xml.dart';

import 'classes.dart';

class Notify extends CuppsBase {
  final NotificationType notificationType;
  final String name;
  Notify({
    required int messageID,
    required String messageName,
    required this.notificationType,
    required this.name,
  }) : super(messageID: messageID, messageName: messageName);

  factory Notify.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageName")
        .value;
    // String xmlns = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "xmlns").value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;
    // String xsi = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "xmlns:xsi").value;
    // String result = document.rootElement.childElements.first.attributes.firstWhere((p0) => p0.name.toString() == "result").value;

    NotificationType notifType = NotificationType.unknown;
    XmlElement notifyElement = document.findAllElements("notify").first;
    String name = "unknown";

    if (notifyElement.attributes
        .any((p0) => p0.name.toString() == "dataAvailable")) {
      if (notifyElement.attributes
              .firstWhere((p0) => p0.name.toString() == "dataAvailable")
              .value ==
          "true") {
        notifType = NotificationType.dataAvailable;
      }
    }
    if (notifyElement.attributes.isNotEmpty) {
      name = notifyElement.attributes.first.name.toString();
    } else {
      name = notifyElement.childElements.first.name.toString();
    }



    if (notifyElement.childElements.any(
        (element) => element.name.toString() == "deviceLockExpiredEvent")) {
      notifType = NotificationType.deviceLockExpire;
    }

    if (notifyElement.childElements.any(
        (element) => element.name.toString() == "deviceStatusNotification")) {
      notifType = NotificationType.deviceStatus;
    }

    notifType = NotificationType.values.firstWhere((element) => element.name==name,orElse:()=> NotificationType.unknown);

    return Notify(
        messageName: messageName,
        messageID: int.parse(messageID),
        notificationType: notifType,
        name: name);
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
    <$messageName"/>
    </cupps>
    ''';
  }

  @override
  String toString() {
    return "$messageID : $messageName : $xsi :  : $xmlns ";
  }
}

enum NotificationType {
  dataAvailable,
  deviceLockExpire,
  deviceStatus,
  applicationAuthenticateEnter,
  applicationAuthenticateExit,
  applicationCotsEnter,
  applicationCotsExit,
  applicationStoppingEnter,
  applicationStoppingExit,
  applicationStartedEnter,
  applicationStartedExit,
  applicationStartingEnter,
  applicationStartingExit,
  applicationStrapEnter,
  applicationStrapExit,
  applicationZombieEnter,
  applicationZombieExit,
  deviceBusyEnter,
  deviceBusyExit,
  deviceErrorEnter,
  deviceErrorExit,
  deviceLockedEnter,
  deviceLockedExit,
  deviceStoppingEnter,
  deviceStoppingExit,
  deviceStartingEnter,
  deviceStartingExit,
  deviceStartedEnter,
  deviceStartedExit,
  deviceStoppedEnter,
  deviceStoppedExit,
  deviceZombieEnter,
  deviceZombieExit,
  cotsApplicationInterfaceEvent,
  dataAvailableNoLockerEvent,
  devicePollingTooFastEvent,
  platformAlertEnter,
  platformAlertExit,
  platformStartedEnter,
  platformStartedExit,
  platformStartingEnter,
  platformStartingExit,
  platformStoppedEnter,
  platformStoppedExit,
  platformStoppingEnter,
  platformStoppingExit,
  userStartedEnter,
  userStartedExit,
  userStartingEnter,
  userStartingExit,
  userStoppedEnter,
  userStoppedExit,
  userStoppingEnter,
  userStoppingExit,
  userSSEnter,
  userSSExit,
  userToEnter,
  userToExit,
  userZombieEnter,
  userZombieExit,
  workstationAlertEnter,
  workstationAlertExit,
  workstationStartedEnter,
  workstationStartedExit,
  workstationStartingEnter,
  workstationStartingExit,
  workstationStoppedEnter,
  workstationStoppedExit,
  workstationStoppingEnter,
  workstationStoppingExit,
  workstationBusyEnter,
  workstationBusyExit,
  workstationZombieEnter,
  workstationZombieExit,
  applicationInstanceBlock,
  exceptionEvent,
  illogicalMessageErrorEvent,
  interfaceLevelErrorEvent,
  logErrorEvent,
  maxMessagesOutErrorEvent,
  messageAccumulationTimeErrorEvent,
  messageResponseTimeErrorEvent,
  printRequestStatusEvent,
  unknown,
}

extension NotifDetails on NotificationType {
  String get name {
    switch (this){
      case NotificationType.dataAvailable:
        return "dataAvailable";
      case NotificationType.deviceStatus:
        return "deviceStatusNotification";
      case NotificationType.applicationAuthenticateEnter:
        return "aAthEnteredEvent";
      case NotificationType.applicationAuthenticateExit:
        return "aAthExitedEvent";
      case NotificationType.applicationCotsEnter:
        return "aCtsEnteredEvent";
      case NotificationType.applicationCotsExit:
        return "aCtsExitedEvent";
      case NotificationType.applicationInstanceBlock:
        return "applicationInstBlockEvent";
      case NotificationType.applicationStoppingEnter:
        return "aSpgEnteredEvent";
      case NotificationType.applicationStoppingExit:
        return "aSpgExitedEvent";
      case NotificationType.applicationStartedEnter:
        return "aStdEnteredEvent";
      case NotificationType.applicationStartedExit:
        return "aStdExitedEvent";
      case NotificationType.applicationStartingEnter:
        return "aStgEnteredEvent";
      case NotificationType.applicationStartingExit:
        return "aStgExitedEvent";
      case NotificationType.applicationStrapEnter:
        return "aStpEnteredEvent";
      case NotificationType.applicationStrapExit:
        return "aStpExitedEvent";
      case NotificationType.applicationZombieEnter:
        return "aZomEnteredEvent";
      case NotificationType.applicationZombieExit:
        return "aZomExitedEvent";
      case NotificationType.cotsApplicationInterfaceEvent:
        return "cotsApplicationInterfaceEvent";
      case NotificationType.dataAvailableNoLockerEvent:
        return "dataAvailableNoLockerEvent";
      case NotificationType.deviceBusyEnter:
        return "dBsyEnteredEvent";
      case NotificationType.deviceBusyExit:
        return "dBsyExitedEvent";
      case NotificationType.deviceErrorEnter:
        return "dErrEnteredEvent";
      case NotificationType.deviceErrorExit:
        return "dErrExitedEvent";
      case NotificationType.deviceLockExpire:
        return "deviceLockExpiredEvent";
      case NotificationType.devicePollingTooFastEvent:
        return "devicePollingTooFastEvent";
      case NotificationType.deviceLockedEnter:
        return "dLkdEnteredEvent";
      case NotificationType.deviceLockedExit:
        return "dLkdExitedEvent";
      case NotificationType.deviceStoppingEnter:
        return "dSpgEnteredEvent";
      case NotificationType.deviceStoppingExit:
        return "dSpgExitedEvent";
      case NotificationType.deviceStartedEnter:
        return "dStdEnteredEvent";
      case NotificationType.deviceStartedExit:
        return "dStdExitedEvent";
      case NotificationType.deviceStartingEnter:
        return "dStgEnteredEvent";
      case NotificationType.deviceStartingExit:
        return "dStgExitedEvent";
      case NotificationType.deviceStoppedEnter:
        return "dStpEnteredEvent";
      case NotificationType.deviceStoppedExit:
        return "dStpExitedEvent";
      case NotificationType.deviceZombieEnter:
        return "dZomEnteredEvent";
      case NotificationType.deviceZombieExit:
        return "dZomExitedEvent";
      case NotificationType.exceptionEvent:
        return "exceptionEvent";
      case NotificationType.illogicalMessageErrorEvent:
        return "illogicalMessageErrorEvent";
      case NotificationType.interfaceLevelErrorEvent:
        return "interfaceLevelErrorEvent";
      case NotificationType.logErrorEvent:
        return "logErrorEvent";
      case NotificationType.maxMessagesOutErrorEvent:
        return "maxMessagesOutErrorEvent";
      case NotificationType.messageAccumulationTimeErrorEvent:
        return "messageAccumulationTimeErrorEvent";
      case NotificationType.messageResponseTimeErrorEvent:
        return "messageResponseTimeErrorEvent";
      case NotificationType.platformAlertEnter:
        return "pAltEnteredEvent";
      case NotificationType.platformAlertExit:
        return "pAltExitedEvent";
      case NotificationType.printRequestStatusEvent:
        return "printRequestStatusEvent";
      case NotificationType.platformStoppingEnter:
        return "pSpgEnteredEvent";
      case NotificationType.platformStoppingExit:
        return "pSpgExitedEvent";
      case NotificationType.platformStartedEnter:
        return "pStdEnteredEvent";
      case NotificationType.platformStartedExit:
        return "pStdExitedEvent";
      case NotificationType.platformStartingEnter:
        return "pStgEnteredEvent";
      case NotificationType.platformStartingExit:
        return "pStgExitedEvent";
      case NotificationType.userStoppingEnter:
        return "uSpgEnteredEvent";
      case NotificationType.userStoppingExit:
        return "uSpgExitedEvent";
      case NotificationType.userSSEnter:
        return "uSsEnteredEvent";
      case NotificationType.userSSExit:
        return "uSsExitedEvent";
      case NotificationType.userStartedEnter:
        return "uStdEnteredEvent";
      case NotificationType.userStartedExit:
        return "uStdExitedEvent";
      case NotificationType.userStartingEnter:
        return "uStgEnteredEvent";
      case NotificationType.userStartingExit:
        return "uStgExitedEvent";
      case NotificationType.userStoppedEnter:
        return "uStpEnteredEvent";
      case NotificationType.userStoppedExit:
        return "uStpExitedEvent";
      case NotificationType.userToEnter:
        return "uToEnteredEvent";
      case NotificationType.userToExit:
        return "uToExitedEvent";
      case NotificationType.userZombieEnter:
        return "uZomEnteredEvent";
      case NotificationType.userZombieExit:
        return "uZomExitedEvent";
      case NotificationType.workstationAlertEnter:
        return "wAltEnteredEvent";
      case NotificationType.workstationAlertExit:
        return "wAltExitedEvent";
      case NotificationType.workstationBusyEnter:
        return "wBsyEnteredEvent";
      case NotificationType.workstationBusyExit:
        return "wBsyExitedEvent";
      case NotificationType.workstationStoppingEnter:
        return "wSpgEnteredEvent";
      case NotificationType.workstationStoppingExit:
        return "wSpgExitedEvent";
      case NotificationType.workstationStartedEnter:
        return "wStdEnteredEvent";
      case NotificationType.workstationStartedExit:
        return "wStdExitedEvent";
      case NotificationType.workstationStartingEnter:
        return "wStgEnteredEvent";
      case NotificationType.workstationStartingExit:
        return "wStgExitedEvent";
      case NotificationType.workstationStoppedEnter:
        return "wStpEnteredEvent";
      case NotificationType.workstationStoppedExit:
        return "wStpExitedEvent";
      case NotificationType.workstationZombieEnter:
        return "wZomEnteredEvent";
      case NotificationType.workstationZombieExit:
        return "wZomExitedEvent";
      default:
        return "unknown";
    }
  }
}



// switch (name){
//   case "dataAvailable":
//     notifType = NotificationType.dataAvailable;
//     break;
//   case "deviceStatusNotification":
//     notifType = NotificationType.deviceStatus;
//     break;
//   case "aAthEnteredEvent":
//     notifType = NotificationType.applicationAuthenticateEnter;
//     break;
//   case "aAthExitedEvent":
//     notifType = NotificationType.applicationAuthenticateExit;
//     break;
//   case "aCtsEnteredEvent":
//     notifType = NotificationType.applicationCotsEnter;
//     break;
//   case "aCtsExitedEvent":
//     notifType = NotificationType.applicationCotsExit;
//     break;
//   case "applicationInstBlockEvent":
//     notifType = NotificationType.applicationInstanceBlock;
//     break;
//   case "aSpgEnteredEvent":
//     notifType = NotificationType.applicationStoppingEnter;
//     break;
//   case "aSpgExitedEvent":
//     notifType = NotificationType.applicationStoppingExit;
//     break;
//   case "aStdEnteredEvent":
//     notifType = NotificationType.applicationStartedEnter;
//     break;
//   case "aStdExitedEvent":
//     notifType = NotificationType.applicationStartedExit;
//     break;
//   case "aStgEnteredEvent":
//     notifType = NotificationType.applicationStartingEnter;
//     break;
//   case "aStgExitedEvent":
//     notifType = NotificationType.applicationStartingExit;
//     break;
//   case "aStpEnteredEvent":
//     notifType = NotificationType.applicationStrapEnter;
//     break;
//   case "aStpExitedEvent":
//     notifType = NotificationType.applicationStrapExit;
//     break;
//   case "aZomEnteredEvent":
//     notifType = NotificationType.applicationZombieEnter;
//     break;
//   case "aZomExitedEvent":
//     notifType = NotificationType.applicationZombieExit;
//     break;
//   case "cotsApplicationInterfaceEvent":
//     notifType = NotificationType.cotsApplicationInterfaceEvent;
//     break;
//   case "dataAvailableNoLockerEvent":
//     notifType = NotificationType.dataAvailableNoLockerEvent;
//     break;
//   case "dBsyEnteredEvent":
//     notifType = NotificationType.deviceBusyEnter;
//     break;
//   case "dBsyExitedEvent":
//     notifType = NotificationType.deviceBusyExit;
//     break;
//   case "dErrEnteredEvent":
//     notifType = NotificationType.deviceErrorEnter;
//     break;
//   case "dErrExitedEvent":
//     notifType = NotificationType.deviceErrorExit;
//     break;
//   case "deviceLockExpiredEvent":
//     notifType = NotificationType.deviceLockExpire;
//     break;
//   case "devicePollingTooFastEvent":
//     notifType = NotificationType.devicePollingTooFastEvent;
//     break;
//   case "dLkdEnteredEvent":
//     notifType = NotificationType.deviceLockedEnter;
//     break;
//   case "dLkdExitedEvent":
//     notifType = NotificationType.deviceLockedExit;
//     break;
//   case "dSpgEnteredEvent":
//     notifType = NotificationType.deviceStoppingEnter;
//     break;
//   case "dSpgExitedEvent":
//     notifType = NotificationType.deviceStoppingExit;
//     break;
//   case "dStdEnteredEvent":
//     notifType = NotificationType.deviceStartedEnter;
//     break;
//   case "dStdExitedEvent":
//     notifType = NotificationType.deviceStartedExit;
//     break;
//   case "dStgEnteredEvent":
//     notifType = NotificationType.deviceStartingEnter;
//     break;
//   case "dStgExitedEvent":
//     notifType = NotificationType.deviceStartingExit;
//     break;
//   case "dStpEnteredEvent":
//     notifType = NotificationType.deviceStoppedEnter;
//     break;
//   case "dStpExitedEvent":
//     notifType = NotificationType.deviceStoppedExit;
//     break;
//   case "dZomEnteredEvent":
//     notifType = NotificationType.deviceZombieEnter;
//     break;
//   case "dZomExitedEvent":
//     notifType = NotificationType.deviceZombieExit;
//     break;
//   case "exceptionEvent":
//     notifType = NotificationType.exceptionEvent;
//     break;
//   case "illogicalMessageErrorEvent":
//     notifType = NotificationType.illogicalMessageErrorEvent;
//     break;
//   case "interfaceLevelErrorEvent":
//     notifType = NotificationType.interfaceLevelErrorEvent;
//     break;
//   case "logErrorEvent":
//     notifType = NotificationType.logErrorEvent;
//     break;
//   case "maxMessagesOutErrorEvent":
//     notifType = NotificationType.maxMessagesOutErrorEvent;
//     break;
//   case "messageAccumulationTimeErrorEvent":
//     notifType = NotificationType.messageAccumulationTimeErrorEvent;
//     break;
//   case "messageResponseTimeErrorEvent":
//     notifType = NotificationType.messageResponseTimeErrorEvent;
//     break;
//   case "pAltEnteredEvent":
//     notifType = NotificationType.platformAlertEnter;
//     break;
//   case "pAltExitedEvent":
//     notifType = NotificationType.platformAlertExit;
//     break;
//   case "printRequestStatusEvent":
//     notifType = NotificationType.printRequestStatusEvent;
//     break;
//   case "pSpgEnteredEvent":
//     notifType = NotificationType.platformStoppingEnter;
//     break;
//   case "pSpgExitedEvent":
//     notifType = NotificationType.platformStoppingExit;
//     break;
//   case "pStdEnteredEvent":
//     notifType = NotificationType.platformStartedEnter;
//     break;
//   case "pStdExitedEvent":
//     notifType = NotificationType.platformStartedExit;
//     break;
//   case "pStgEnteredEvent":
//     notifType = NotificationType.platformStartingEnter;
//     break;
//   case "pStgExitedEvent":
//     notifType = NotificationType.platformStartingExit;
//     break;
//   case "uSpgEnteredEvent":
//     notifType = NotificationType.userStoppingEnter;
//     break;
//   case "uSpgExitedEvent":
//     notifType = NotificationType.userStoppingExit;
//     break;
//   case "uSsEnteredEvent":
//     notifType = NotificationType.userSSEnter;
//     break;
//   case "uSsExitedEvent":
//     notifType = NotificationType.userSSExit;
//     break;
//   case "uStdEnteredEvent":
//     notifType = NotificationType.userStartedEnter;
//     break;
//   case "uStdExitedEvent":
//     notifType = NotificationType.userStartedExit;
//     break;
//   case "uStgEnteredEvent":
//     notifType = NotificationType.userStartingEnter;
//     break;
//   case "uStgExitedEvent":
//     notifType = NotificationType.userStartingExit;
//     break;
//   case "uStpEnteredEvent":
//     notifType = NotificationType.userStoppedEnter;
//     break;
//   case "uStpExitedEvent":
//     notifType = NotificationType.userStoppedExit;
//     break;
//   case "uToEnteredEvent":
//     notifType = NotificationType.userToEnter;
//     break;
//   case "uToExitedEvent":
//     notifType = NotificationType.userToExit;
//     break;
//   case "uZomEnteredEvent":
//     notifType = NotificationType.userZombieEnter;
//     break;
//   case "uZomExitedEvent":
//     notifType = NotificationType.userZombieExit;
//     break;
//   case "wAltEnteredEvent":
//     notifType = NotificationType.workstationAlertEnter;
//     break;
//   case "wAltExitedEvent":
//     notifType = NotificationType.workstationAlertExit;
//     break;
//   case "wBsyEnteredEvent":
//     notifType = NotificationType.workstationBusyEnter;
//     break;
//   case "wBsyExitedEvent":
//     notifType = NotificationType.workstationBusyExit;
//     break;
//   case "wSpgEnteredEvent":
//     notifType = NotificationType.workstationStoppingEnter;
//     break;
//   case "wSpgExitedEvent":
//     notifType = NotificationType.workstationStoppingExit;
//     break;
//   case "wStdEnteredEvent":
//     notifType = NotificationType.workstationStartedEnter;
//     break;
//   case "wStdExitedEvent":
//     notifType = NotificationType.workstationStartedExit;
//     break;
//   case "wStgEnteredEvent":
//     notifType = NotificationType.workstationStartingEnter;
//     break;
//   case "wStgExitedEvent":
//     notifType = NotificationType.workstationStartingExit;
//     break;
//   case "wStpEnteredEvent":
//     notifType = NotificationType.workstationStoppedEnter;
//     break;
//   case "wStpExitedEvent":
//     notifType = NotificationType.workstationStoppedExit;
//     break;
//   case "wZomEnteredEvent":
//     notifType = NotificationType.workstationZombieEnter;
//     break;
//   case "wZomExitedEvent":
//     notifType = NotificationType.workstationZombieExit;
//     break;
//   default:
//     notifType = NotificationType.unknown;
// }